==> models_weebi-1.1.2.tar.gz: lib/src/models/article/article_basket.dart <==
import 'dart:convert';

import 'package:mobx/mobx.dart';
import 'package:models_weebi/abstracts.dart';
import 'package:models_weebi/src/code_related_abstract_classes/default_attributes.dart';
import 'package:models_weebi/src/models/article/basket_logic/markup_and_discount.dart';

import 'package:models_weebi/src/models/article/basket_logic/proxy_article_worth.dart';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/models.dart' show ProxyArticle, Calibre;
import 'package:collection/collection.dart';

extension GetProxiesListWithPriceAndCostArticleNotCreatedYetOnlyExt
    on List<ProxyArticle> {
  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in this) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }
}

class ArticleBasketOnTicket extends ArticleBasketRawAbstract
    implements Discount, Markup {
  Iterable<ProxyArticleWorth> getProxiesListWithPriceAndCost(
      Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  @override
  final num discountAmountSalesOnly;
  @override
  final num markupAmountSalesOnly;

  ArticleBasketOnTicket({
    required super.calibreId,
    required super.id,
    super.kind = ArticleKind.basket,
    required super.designation,
    super.status = true,
    required super.proxies,
    this.discountAmountSalesOnly = 0,
    this.markupAmountSalesOnly = 0,
  });

  static final dummyBasketOnTicket = ArticleBasketOnTicket(
      calibreId: 2,
      id: 1,
      designation: 'basket dummy',
      proxies: [
        ProxyArticle(
            calibreId: 2,
            articleId: 1,
            id: 1,
            proxyCalibreId: 1,
            proxyArticleId: 1,
            minimumUnitPerBasket: 1,
            articleWeight: 1)
      ]);

  factory ArticleBasketOnTicket.fromMap(Map<String, dynamic> map) {
    return ArticleBasketOnTicket(
      calibreId: map['calibreId'] as int,
      id: map['id'] as int,
      designation: map['designation'] ?? (map['fullName'] ?? ''),
      kind: ArticleKind.basket,
      proxies: map['proxies'] != null
          ? List<ProxyArticle>.from(
              map['proxies']?.map((x) => ProxyArticle.fromMap(x)))
          : [],
      discountAmountSalesOnly:
          map['markupAmount'] == null ? 0 : (map['discountAmount'] as num),
      markupAmountSalesOnly:
          map['markupAmount'] == null ? 0 : (map['markupAmount'] as num),
    );
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'calibreId': calibreId,
      'id': id,
      'designation': designation,
      'proxies': proxies.map((x) => x.toMap()).toList(),
      'discountAmount': discountAmountSalesOnly,
      'markupAmount': markupAmountSalesOnly,
    };
  }

  @override
  String toString() {
    return """
ArticleBasket(
  calibreId: $calibreId,
  id: $id,
  designation: '$designation',
  discountAmount: $discountAmountSalesOnly,
  markupAmount: $markupAmountSalesOnly,
  proxies: $proxies,
  )""";
  }

  factory ArticleBasketOnTicket.fromJson(String source) =>
      ArticleBasketOnTicket.fromMap(json.decode(source));

  ArticleBasketOnTicket copyWith({
    int? calibreId,
    int? id,
    String? designation,
    num? discountAmountSalesOnly,
    num? markupAmountSalesOnly,
    List<ProxyArticle>? proxies,
  }) {
    return ArticleBasketOnTicket(
      calibreId: calibreId ?? this.calibreId,
      id: id ?? this.id,
      designation: designation ?? this.designation,
      discountAmountSalesOnly:
          discountAmountSalesOnly ?? this.discountAmountSalesOnly,
      markupAmountSalesOnly:
          markupAmountSalesOnly ?? this.markupAmountSalesOnly,
      proxies: proxies ?? this.proxies,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;
    return other is ArticleBasketOnTicket &&
        other.calibreId == calibreId &&
        other.id == id &&
        listEquals(other.proxies, proxies);
  }

  @override
  int get hashCode => id.hashCode ^ calibreId.hashCode;
}

class ArticleBasket extends ArticleBasketRawAbstract
    implements Discount, Markup, DefaultDates, ArticleCode {
  // article price and cost can change
  // so proxies only save ref not price / nor cost which are fetched when invoked

  @override
  final num discountAmountSalesOnly;
  @override
  final num markupAmountSalesOnly;
  @override
  final int? articleCode;

  @override
  final DateTime creationDate;
  @override
  final DateTime? updateDate;
  @override
  final DateTime? statusUpdateDate;
  ArticleBasket({
    required super.calibreId,
    required super.id,
    required super.designation,
    required super.proxies,
    super.kind = ArticleKind.basket,
    this.articleCode,
    required this.creationDate,
    this.updateDate,
    this.statusUpdateDate,
    @observable super.status = true,
    this.discountAmountSalesOnly = 0,
    this.markupAmountSalesOnly = 0,
  });

  Iterable<ProxyArticleWorth> getProxiesListWithPriceAndCost(
      Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  ArticleBasketOnTicket get articleBasketOnTicket => ArticleBasketOnTicket(
      calibreId: calibreId, id: id, designation: designation, proxies: proxies);

  static ArticleBasket get dummy => ArticleBasket(
        calibreId: 2,
        id: 1,
        designation: 'dummy',
        articleCode: 1,
        creationDate: DatesWeebi.defaultDate,
        updateDate: DatesWeebi.defaultDate,
        status: true,
        proxies: [ProxyArticle.dummy],
        statusUpdateDate: DatesWeebi.defaultDate,
        discountAmountSalesOnly: 0,
      );

  factory ArticleBasket.fromMap(Map<String, dynamic> map) {
    return ArticleBasket(
        calibreId: map['calibreId'] != null
            ? map['calibreId'] as int
            : map['lineId'] != null
                ? map['lineId'] as int
                : map['productId'] as int,
        id: map['id'] as int,
        designation: map['designation'] == null
            ? map['fullName'] as String
            : map['designation'] as String,
        kind: ArticleKind.basket,
        articleCode: map['articleCode'] ?? 0,
        discountAmountSalesOnly: (map['discountAmount'] as num),
        markupAmountSalesOnly:
            map['markupAmount'] == null ? 0 : (map['markupAmount'] as num),
        creationDate: map['creationDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['creationDate']),
        updateDate: map['updateDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['updateDate']),
        proxies: map['proxies'] != null
            ? List<ProxyArticle>.from(
                map['proxies']?.map((x) => ProxyArticle.fromMap(x)))
            : [],
        statusUpdateDate: map['statusUpdateDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['statusUpdateDate']),
        status: map['status']);
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'proxies': proxies.map((x) => x.toMap()).toList(),
      'calibreId': calibreId,
      'id': id,
      'discountAmount': discountAmountSalesOnly,
      'markupAmount': markupAmountSalesOnly,
      'designation': designation,
      'articleCode': articleCode ?? 0,
      'creationDate': creationDate.toIso8601String(),
      'updateDate': updateDate?.toIso8601String(),
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'status': status,
    };
  }

  @override
  String toString() {
    return """
ArticleBasket(
  calibreId: $calibreId,
  id: $id,
  designation: '$designation',
  articleCode: $articleCode,
  discountAmount: $discountAmountSalesOnly,
  markupAmount: $markupAmountSalesOnly,
  creationDate: $creationDate,
  updateDate: $updateDate,
  statusUpdateDate: $statusUpdateDate,
  status: $status,
  proxies: $proxies,
  )""";
  }

  factory ArticleBasket.fromJson(String source) =>
      ArticleBasket.fromMap(json.decode(source));

  ArticleBasket copyWith({
    int? calibreId,
    int? id,
    String? designation,
    int? articleCode,
    num? discountAmountSalesOnly,
    num? markupAmountSalesOnly,
    DateTime? creationDate,
    DateTime? updateDate,
    bool? status,
    List<ProxyArticle>? proxies,
    DateTime? statusUpdateDate,
  }) {
    return ArticleBasket(
      calibreId: calibreId ?? this.calibreId,
      id: id ?? this.id,
      designation: designation ?? this.designation,
      articleCode: articleCode ?? this.articleCode,
      discountAmountSalesOnly:
          discountAmountSalesOnly ?? this.discountAmountSalesOnly,
      markupAmountSalesOnly:
          markupAmountSalesOnly ?? this.markupAmountSalesOnly,
      creationDate: creationDate ?? this.creationDate,
      updateDate: updateDate ?? this.updateDate,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      status: status ?? this.status,
      proxies: proxies ?? this.proxies,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;
    return other is ArticleBasket &&
        other.calibreId == calibreId &&
        other.id == id &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate &&
        listEquals(other.proxies, proxies);
  }

  @override
  int get hashCode => id.hashCode ^ calibreId.hashCode;
}

==> models_weebi-1.1.3.tar.gz: lib/src/models/article/article_basket.dart <==
import 'dart:convert';

import 'package:mobx/mobx.dart';
import 'package:models_weebi/abstracts.dart';
import 'package:models_weebi/src/code_related_abstract_classes/default_attributes.dart';
import 'package:models_weebi/src/models/article/basket_logic/markup_and_discount.dart';

import 'package:models_weebi/src/models/article/basket_logic/proxy_article_worth.dart';
import 'package:models_weebi/utils.dart';
import 'package:models_weebi/models.dart' show ProxyArticle, Calibre;
import 'package:collection/collection.dart';

extension GetProxiesListWithPriceAndCostArticleNotCreatedYetOnlyExt
    on List<ProxyArticle> {
  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in this) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }
}

class ArticleBasketOnTicket extends ArticleBasketRawAbstract
    implements Discount, Markup {
  Iterable<ProxyArticleWorth> getProxiesListWithPriceAndCost(
      Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  @override
  final num discountAmountSalesOnly;
  @override
  final num markupAmountSalesOnly;

  ArticleBasketOnTicket({
    required super.calibreId,
    required super.id,
    super.kind = ArticleKind.basket,
    required super.designation,
    super.status = true,
    required super.proxies,
    this.discountAmountSalesOnly = 0,
    this.markupAmountSalesOnly = 0,
  });

  static final dummyBasketOnTicket = ArticleBasketOnTicket(
      calibreId: 2,
      id: 1,
      designation: 'basket dummy',
      proxies: [
        ProxyArticle(
            calibreId: 2,
            articleId: 1,
            id: 1,
            proxyCalibreId: 1,
            proxyArticleId: 1,
            minimumUnitPerBasket: 1,
            articleWeight: 1)
      ]);

  factory ArticleBasketOnTicket.fromMap(Map<String, dynamic> map) {
    return ArticleBasketOnTicket(
      calibreId: map['calibreId'] != null
          ? map['calibreId'] as int
          : map['lineId'] != null
              ? map['lineId'] as int
              : map['productId'] as int,
      id: map['id'] as int,
      designation: map['designation'] == null
          ? map['fullName'] as String
          : map['designation'] as String,
      kind: ArticleKind.basket,
      proxies: map['proxies'] != null
          ? List<ProxyArticle>.from(
              map['proxies']?.map((x) => ProxyArticle.fromMap(x)))
          : [],
      discountAmountSalesOnly:
          map['markupAmount'] == null ? 0 : (map['discountAmount'] as num),
      markupAmountSalesOnly:
          map['markupAmount'] == null ? 0 : (map['markupAmount'] as num),
    );
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'calibreId': calibreId,
      'id': id,
      'designation': designation,
      'proxies': proxies.map((x) => x.toMap()).toList(),
      'discountAmount': discountAmountSalesOnly,
      'markupAmount': markupAmountSalesOnly,
    };
  }

  @override
  String toString() {
    return """
ArticleBasket(
  calibreId: $calibreId,
  id: $id,
  designation: '$designation',
  discountAmount: $discountAmountSalesOnly,
  markupAmount: $markupAmountSalesOnly,
  proxies: $proxies,
  )""";
  }

  factory ArticleBasketOnTicket.fromJson(String source) =>
      ArticleBasketOnTicket.fromMap(json.decode(source));

  ArticleBasketOnTicket copyWith({
    int? calibreId,
    int? id,
    String? designation,
    num? discountAmountSalesOnly,
    num? markupAmountSalesOnly,
    List<ProxyArticle>? proxies,
  }) {
    return ArticleBasketOnTicket(
      calibreId: calibreId ?? this.calibreId,
      id: id ?? this.id,
      designation: designation ?? this.designation,
      discountAmountSalesOnly:
          discountAmountSalesOnly ?? this.discountAmountSalesOnly,
      markupAmountSalesOnly:
          markupAmountSalesOnly ?? this.markupAmountSalesOnly,
      proxies: proxies ?? this.proxies,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;
    return other is ArticleBasketOnTicket &&
        other.calibreId == calibreId &&
        other.id == id &&
        listEquals(other.proxies, proxies);
  }

  @override
  int get hashCode => id.hashCode ^ calibreId.hashCode;
}

class ArticleBasket extends ArticleBasketRawAbstract
    implements Discount, Markup, DefaultDates, ArticleCode {
  // article price and cost can change
  // so proxies only save ref not price / nor cost which are fetched when invoked

  @override
  final num discountAmountSalesOnly;
  @override
  final num markupAmountSalesOnly;
  @override
  final int? articleCode;

  @override
  final DateTime creationDate;
  @override
  final DateTime? updateDate;
  @override
  final DateTime? statusUpdateDate;
  ArticleBasket({
    required super.calibreId,
    required super.id,
    required super.designation,
    required super.proxies,
    super.kind = ArticleKind.basket,
    this.articleCode,
    required this.creationDate,
    this.updateDate,
    this.statusUpdateDate,
    @observable super.status = true,
    this.discountAmountSalesOnly = 0,
    this.markupAmountSalesOnly = 0,
  });

  Iterable<ProxyArticleWorth> getProxiesListWithPriceAndCost(
      Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  Iterable<ProxyArticleWorth>
      getProxiesListWithPriceAndCostArticleNotCreatedYetOnly(
          Iterable<Calibre> calibres) {
    final proxiesWorth = <ProxyArticleWorth>[];
    for (final p in proxies) {
      final temp = p.getProxyArticleWorth(calibres);
      proxiesWorth.add(temp);
    }
    return proxiesWorth;
  }

  ArticleBasketOnTicket get articleBasketOnTicket => ArticleBasketOnTicket(
      calibreId: calibreId, id: id, designation: designation, proxies: proxies);

  static ArticleBasket get dummy => ArticleBasket(
        calibreId: 2,
        id: 1,
        designation: 'dummy',
        articleCode: 1,
        creationDate: DatesWeebi.defaultDate,
        updateDate: DatesWeebi.defaultDate,
        status: true,
        proxies: [ProxyArticle.dummy],
        statusUpdateDate: DatesWeebi.defaultDate,
        discountAmountSalesOnly: 0,
      );

  factory ArticleBasket.fromMap(Map<String, dynamic> map) {
    return ArticleBasket(
        calibreId: map['calibreId'] != null
            ? map['calibreId'] as int
            : map['lineId'] != null
                ? map['lineId'] as int
                : map['productId'] as int,
        designation: map['designation'] == null
            ? map['fullName'] as String
            : map['designation'] as String,
        id: map['id'] as int,
        kind: ArticleKind.basket,
        articleCode: map['articleCode'] ?? 0,
        discountAmountSalesOnly:
            map['discountAmount'] == null ? 0 : (map['discountAmount'] as num),
        markupAmountSalesOnly:
            map['markupAmount'] == null ? 0 : (map['markupAmount'] as num),
        creationDate: map['creationDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['creationDate']),
        updateDate: map['updateDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['updateDate']),
        proxies: map['proxies'] != null
            ? List<ProxyArticle>.from(
                map['proxies']?.map((x) => ProxyArticle.fromMap(x)))
            : [],
        statusUpdateDate: map['statusUpdateDate'] == null
            ? DatesWeebi.defaultDate
            : DateTime.parse(map['statusUpdateDate']),
        status: map['status']);
  }

  @override
  String toJson() => json.encode(toMap());

  @override
  Map<String, dynamic> toMap() {
    return {
      'proxies': proxies.map((x) => x.toMap()).toList(),
      'calibreId': calibreId,
      'id': id,
      'discountAmount': discountAmountSalesOnly,
      'markupAmount': markupAmountSalesOnly,
      'designation': designation,
      'articleCode': articleCode ?? 0,
      'creationDate': creationDate.toIso8601String(),
      'updateDate': updateDate?.toIso8601String(),
      'statusUpdateDate': statusUpdateDate?.toIso8601String(),
      'status': status,
    };
  }

  @override
  String toString() {
    return """
ArticleBasket(
  calibreId: $calibreId,
  id: $id,
  designation: '$designation',
  articleCode: $articleCode,
  discountAmount: $discountAmountSalesOnly,
  markupAmount: $markupAmountSalesOnly,
  creationDate: $creationDate,
  updateDate: $updateDate,
  statusUpdateDate: $statusUpdateDate,
  status: $status,
  proxies: $proxies,
  )""";
  }

  factory ArticleBasket.fromJson(String source) =>
      ArticleBasket.fromMap(json.decode(source));

  ArticleBasket copyWith({
    int? calibreId,
    int? id,
    String? designation,
    int? articleCode,
    num? discountAmountSalesOnly,
    num? markupAmountSalesOnly,
    DateTime? creationDate,
    DateTime? updateDate,
    bool? status,
    List<ProxyArticle>? proxies,
    DateTime? statusUpdateDate,
  }) {
    return ArticleBasket(
      calibreId: calibreId ?? this.calibreId,
      id: id ?? this.id,
      designation: designation ?? this.designation,
      articleCode: articleCode ?? this.articleCode,
      discountAmountSalesOnly:
          discountAmountSalesOnly ?? this.discountAmountSalesOnly,
      markupAmountSalesOnly:
          markupAmountSalesOnly ?? this.markupAmountSalesOnly,
      creationDate: creationDate ?? this.creationDate,
      updateDate: updateDate ?? this.updateDate,
      statusUpdateDate: statusUpdateDate ?? this.statusUpdateDate,
      status: status ?? this.status,
      proxies: proxies ?? this.proxies,
    );
  }

  @override
  bool operator ==(Object other) {
    if (identical(this, other)) return true;
    final listEquals = const DeepCollectionEquality().equals;
    return other is ArticleBasket &&
        other.calibreId == calibreId &&
        other.id == id &&
        other.creationDate == creationDate &&
        other.updateDate == updateDate &&
        listEquals(other.proxies, proxies);
  }

  @override
  int get hashCode => id.hashCode ^ calibreId.hashCode;
}
